﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Security.Cryptography;
using Windows.Security.Cryptography.Core;
using Windows.Storage.Streams;

namespace SignApp
{
    /// <summary>
    /// 可用于自身或导航至 Frame 内部的空白页。
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }


        #region 私有变量
        /// <summary>
        /// 公钥
        /// </summary>
        IBuffer publicKey = null;
        /// <summary>
        /// 签名得到的数据，在验证时用到
        /// </summary>
        IBuffer signedData = null;
        #endregion

        private void OnSign(object sender, RoutedEventArgs e)
        {
            // 使用特定算法获取相应的对象
            AsymmetricKeyAlgorithmProvider asymt = AsymmetricKeyAlgorithmProvider.OpenAlgorithm(AsymmetricAlgorithmNames.DsaSha1);
            // 生成公/私密钥对
            CryptographicKey key = asymt.CreateKeyPair(512);
            // 导出公钥
            publicKey = key.ExportPublicKey();
            // 对数据进行签名
            IBuffer data = CryptographicBuffer.ConvertStringToBinary(txtInput.Text.Trim(), BinaryStringEncoding.Utf8);
            signedData = CryptographicEngine.Sign(key, data);

            // 显示结果
            tbPublicKey.Text = $"公钥：{CryptographicBuffer.EncodeToHexString(publicKey)}";
            tbSigndata.Text = $"答名后：{CryptographicBuffer.EncodeToHexString(signedData)}";
        }

        private void OnVerify(object sender, RoutedEventArgs e)
        {
            // 同样，先获取操作对象
            // 注意算法要匹配，不要乱来
            AsymmetricKeyAlgorithmProvider asmt = AsymmetricKeyAlgorithmProvider.OpenAlgorithm(AsymmetricAlgorithmNames.DsaSha1);

            // 导入公钥
            CryptographicKey key = asmt.ImportPublicKey(publicKey);
            
            // 开始验证
            IBuffer data = CryptographicBuffer.ConvertStringToBinary(txtVerifyInput.Text.Trim(), BinaryStringEncoding.Utf8);
            // 注意，第三个参数是你刚才签名后得到的数据
            bool b = CryptographicEngine.VerifySignature(key, data, signedData);
            // 显示结果
            tbVerRes.Text = (b == true) ? "验证成功。" : "验证失败。";
        }
    }
}
